package virtual;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.text.InternationalFormatter;
import javax.swing.text.MaskFormatter;

import utils.Global;
import gui.CanvasDialogue;
import editor.FieldPictoConfirm;

/**
 * Builds the button configuration dialogue from the pictogram selection dialogue.
 * @author njerez, hqcasanova
 */
public class NuevoBoton extends CanvasDialogue implements ActionListener {
    private FieldPictoConfirm eligePicto;	//pictogram selection dialogue
    
    /**
     * Customised constructor. Sets up both the button configuration and the pictogram
     * selection dialogues. The latter will be added to the "new button" dialogue
     * @param title Title for the button configuration dialogue
     * @param owner Owner of the button configuration dialogue
     * @author hqcasanova
     */
	public NuevoBoton(String title, Window owner) {
		super (title, owner, null, null, null, false, JFrame.DISPOSE_ON_CLOSE, Global.FAVICON_TV, true);
		
		//Builds the pictogram selection dialogue
		eligePicto = new FieldPictoConfirm (null, Global.LANG.getString("buscapict"), 
				                              Global.LANG.getString("selecpict"),
				                              Global.BUTTONNAME_REGEX);
		
		//Removes the bottom panel with the confirmation buttons
		eligePicto.removeBottomPanel();		
	}
	
	/**
     * Retrieves the id of the selected pictogram on the button configuration dialogue
     * @param sDefaultPictoId Default pictogram id if there is no selection
     * @return Selected pictogram's id
     * @author hqcasanova
     */
    protected String getPictoId (String sDefaultPictoId) {
    	return eligePicto.getPictoId (sDefaultPictoId);
    }   
	
	/**
	 * @author njerez
	 * Deshabilita el lienzo. O esa era la intención
	 * Por ahora lo que hace es eliminarlo directamente.
	 */
	public void deshabilitaPictos (boolean bvalor) {
		eligePicto.getCanvas().setVisible (!bvalor);
	}
    
    /* ....................................................................... */
    /* ...........................  Variables para   ......................... */
    /* ........................  Ventana-Configurador   ...................... */
    /* ...........................  @autor njerez    ......................... */
    /* ....................................................................... */
    // static ->  la variable es compartida y solo hay una para todas las instancias de la clase. Si no pones static, cada instancia tiene la suya propia.
    // final  ->  si la inicializas al declararla, ya no puedes cambiarle el valor.
    private String sSupr, sAceptar,sCancelar, sTipoTecla, sITexto, sColorTexto, sColorBack, sAncho, sLargo, sFuente;
    private JButton jBAceptar, jBSupr, jBTexto, jBBack, jBCancelar;
    private JLabel jLTipoTecla, jLTipoTeclaP, jLTexto, jLColorTexto, jLColorBack, jAncho, jLargo, jFuente;
    private JTextField jTTexto; //, jTAncho, jTLargo;
    private JFormattedTextField  jTLargo, jTAncho;
    private JRadioButton jRTipoTexto, jRTipoPicto;
    private ButtonGroup BTipo;
    private JPanel jPanel1, jPanel2;
    private Color cTexto, cBack;
    private boolean bSupr;
    private int idBoton;
    private JComboBox jFuenteList;
    private String[] sfont = { Global.LANG.getString("flittle")+LITTLE, Global.LANG.getString("fnormal")+NORMAL, Global.LANG.getString("fbig")+BIG };
    public static final int LITTLE = 12, NORMAL = 16, BIG = 20;
    public static final String sTexto = Global.LANG.getString("text"), sPicto = Global.LANG.getString("pict");
    private MaskFormatter mascara;
    
    /****
     * FUNCI�N SETVALORESVENTANA
	 * Utilizando los atributos del JButton que se le pasa como 
     * parametro, este procedimiento cambia el estilo inicial de la ventana
     * y por defecto introduce el valor que posee el JButton para cada 
     * atributo.
     * 
     *  @param jboton: Boton del que se van a utilizar los valores.
     *  @author njerez
     * 
 **********************************************************************************/
    public void setValoresVentana (JButton jBoton){
        
    	jBTexto.setContentAreaFilled (false);
    	jBTexto.setOpaque (true);
    	jBTexto.setBackground(jBoton.getForeground());
    	jBBack.setContentAreaFilled (false);
    	jBBack.setOpaque (true);
    	jBBack.setBackground(jBoton.getBackground());
    	jBTexto.setBorder (BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
    	jBBack.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
    	
    	jTTexto.setText(jBoton.getText());
    	jTTexto.setColumns(20); // Define tamaño maximo.
    	String sPicto = jBoton.getName();
    	getWindow().getContentPane().add (eligePicto.getCanvas(), BorderLayout.CENTER);
    	
    	if (sPicto !=  null){// es tipo pictograma
    		jTTexto.setText(jBoton.getToolTipText()); // si es pictograma el texto lo tiene en el tooltip
    		jRTipoTexto.setSelected(false);
    		jRTipoPicto.setSelected(true);
    		deshabilitaPictos (false);
    	}
    	else {   // es tipo texto
    		jRTipoTexto.setSelected(true);
    		jRTipoPicto.setSelected(false);
    		deshabilitaPictos (true);
    	}
    	//getVentana().repaint();
    	cTexto = jBoton.getForeground();
    	cBack = jBoton.getBackground();
    	//bCambiaColorT = false; // esto se usa???????
    	//bCambiaColorB = false;  // esto se usa???????
    	int itam = jBoton.getFont().getSize();
    	if (itam == LITTLE){ jFuenteList.setSelectedIndex(0);}
    	else if (itam == BIG){ jFuenteList.setSelectedIndex(2);}
    	else { jFuenteList.setSelectedIndex(1);}
    	
    	jTAncho.setText (Integer.toString(jBoton.getWidth()));
    	jTLargo.setText (Integer.toString(jBoton.getHeight())); 
        bSupr = false;
    	
    }
    
 /****
     * FUNCION GETCOLORTEXTO
     * Devuelve el Color del Texto seleccionado por el usuario en la ventana.
     * @return Color: Color del Texto.
     * @author njerez
 **********************************************************************************/
    public Color getColorTexto() {  
    	return cTexto; 
    }

 /****
     * FUNCION GETCOLORBACK
     * Devuelve el Color de Fondo seleccionado por el usuario en la ventana.
     * @return Color: Color del Fondo.
     * @author njerez
 **********************************************************************************/
    public Color getColorBack() {  
    	return cBack;  
    }

 /****
     * FUNCION GETTEXTO
     * Devuelve el texto introducido por el usuario en el campo de la ventana.
     * @return String: Texto introducido
     * @author njerez
 **********************************************************************************/
    public String getTexto() {    return jTTexto.getText();    }

 /****
     * FUNCION GETTIPO
     * Devuelve el tipo de tecla seleccionado por el usuario. Puede ser tipo texto
     * o tipo pictograma
     * @return String: T si es tipo texto, P si es tipo pictograma.
     * @author njerez
 **********************************************************************************/
    public String getTipo() {
        if (jRTipoTexto.isSelected()) {
            return "T";
        } else {
            return "P";
        }
    }
    
 /****
     * FUNCION ISPARSABLETOINT
     * Comprueba si el string pasado por par�metro se puede convertir en un entero.
     * @return boolean: True si es parseable, False en caso contrario
     * @author njerez
 **********************************************************************************/
    public boolean isParsableToInt(String i)
    {
    	try
    	{
    			Integer.parseInt(i);
    			return true;
    	}
    	catch(NumberFormatException nfe)
    	{
    		return false;
    	}
    }
    
 /****
     * FUNCION GETANCHO
     * Devuelve el ancho introducido por el usuario si �ste es un entero.
     * @return int: devuelve el ancho si es valor v�lido, sino devuelve -1
     * @author njerez
 **********************************************************************************/
    public int getAncho() {
     	if (isParsableToInt(jTAncho.getText())){ return Integer.parseInt(jTAncho.getText()); }	
     	else { return -1;}
    }
    
 /****
     * FUNCION GETLARGO
     * Devuelve el largo/alto introducido por el usuario si �ste es un entero.
     * @return int: devuelve el alto si es valor v�lido, sino devuelve -1
     * @author njerez
 **********************************************************************************/
    public int getLargo() {
    	if (isParsableToInt(jTLargo.getText())){ return Integer.parseInt(jTLargo.getText()); }	
     	else { return -1;}
    }
    
 /****
     * FUNCI�N SETCOLORES
     * Asigna los colores pasados por par�metro a los colores de texto y de fondo
     * @param cTextop: color a asignar para el texto
     * @param cBackp: color a asignar para el fondo
     * @author njerez
 **********************************************************************************/ 
    public void setColors(Color cTextop, Color cBackp) {
    	cTexto = cTextop;
        cBack = cBackp;   	
    }
    
 /****
     * FUNCI�N SETIDBOTON
     * Asigna el iD del bot�n que llam� a este objeto.
     * @param iD: entero que identifica al bot�n dentro del array de botones del
     * tecladovirtual.
     * @author njerez
 **********************************************************************************/    
    public void setIdBoton(int iD) {     
    	idBoton = iD;   
    	
    }
    
 /****
     * FUNCI�N GETIDBOTON
     * Devuelve el iD del bot�n que llam� a este objeto.
     * @return int: entero que identifica al bot�n dentro del array de botones del
     * tecladovirtual.
     * @author njerez
 **********************************************************************************/   
    public int getIdBoton() {
        return idBoton;
    }
    
 /****
     * FUNCI�N GETFUENTE
     * Devuelve el tama�o de fuente introducido por el usuario, que puede ser LITTlE
     * NORMAL o BIG. Si no se ha seleccionado ninguno, por defecto ser� NORMAL.
     * @return int: tama�o de fuente 
     * @author njerez
 **********************************************************************************/   
    public int getFuente() {
    	int id = jFuenteList.getSelectedIndex();
    	int itam = NORMAL;    	
    	if (id == 0){itam=LITTLE;}
    	if (id == 1){itam=NORMAL;}
    	if (id == 2){itam=BIG;}
    	return itam;
    }
    
 /****
     * FUNCI�N ISTEXTO
     * Devuelve si al tipo seleccionado para la tecla es tipo texto o no.
     * @return boolean: Devuelve true si es tipo texto, false si es tipo pictograma
     * @author njerez
 **********************************************************************************/   
    public boolean isTexto(){
        return jRTipoTexto.isSelected();
    }
 
 /****
     * FUNCI�N ISSUPRIMED
     * Devuelve si el usuario ha seleccionado la opci�n de suprimir la tecla
     * @return boolean: Devuelve true si es la tecla se desea eliminar, false en
     * case contrario
     * @author njerez
 **********************************************************************************/   
    public boolean isSuprimed(){
        return bSupr;
    }
   
 /****
     * FUNCIÓN ACTIONPERFORMED
     * Dependiendo de si el tipo de tecla seleccioando por el usuario es tipo picto
     * o tipo texto, muestra o no muestra el lienzo. El lienzo es el buscador de
     * pictogramas.Despu�s repinta la ventana para mostrar los cambios.
     * @param e: Evento
     * @author njerez
     * TODO: Needs modifying. Window refresh doesn't work properly because of 
     * unorthodox window closing strategy (can't do a "setVisible(false)"). 
 **********************************************************************************/   
   public void actionPerformed (ActionEvent e) {
	// 
	    if (e.getActionCommand() == sTexto){
    	          deshabilitaPictos (true);
    	          refresh();    	          
        } else if (e.getActionCommand() == sPicto){
    	          deshabilitaPictos (false);
    	          refresh();
        }      
    }
  
/****
    * FUNCI�N INITSTRING
	* Inicializa los strings y variables bSupr utilizadas en la clase.
    * @author njerez
**********************************************************************************/   
    private void initStrings (){
        bSupr = false;
        sTipoTecla = Global.LANG.getString("sTipoTecla");
        sITexto = Global.LANG.getString("sITexto");
        sColorTexto = Global.LANG.getString("sColorTexto");
        sColorBack = Global.LANG.getString("sColorBack");
        sAncho = Global.LANG.getString("sAncho");
        sLargo = Global.LANG.getString("sLargo");
        sFuente = Global.LANG.getString("sFuente");
        sAceptar = Global.LANG.getString("sAceptar");
        sSupr = Global.LANG.getString("sSupr"); 
        sCancelar = Global.LANG.getString("sCancelar");
    }
  
 /****
     * FUNCI�N INITCOMPONENTS
 	 * Inicializa los componentes b�sicos de la clase (ventanaTitulo) utilizada
 	 * como configurador de tecla para la clase TecladoVirtual.
     * @author njerez
 **********************************************************************************/   
    public void initComponents() {
    	    	
        initStrings();
        setBotonesConf();
        setFuenteList();
        setBotonesColor();
        setTextFields();
        setGrupoTipo();
        setPanels();
        refresh();

    }//fin initcomponents
  
    
 /****
     * FUNCI�N setPanels
 	 * Crea los dos paneles y les asigna los componentes creados.
 	 * Luego mete estos dos paneles en la ventana con el lienzo para 
 	 * la b�squeda de pictoramas.
 	 * PAra la ventana cuya finalidad es ser el configurador de teclas para 
 	 * tecladovirtual.
     * @author njerez
 **********************************************************************************/
    private void setPanels(){
    	 jPanel1 = new JPanel();
         jPanel1.setLayout(new GridLayout(5, 2, 10, 5));
         jPanel1.setBorder (BorderFactory.createEmptyBorder (15, 15, 10, 15));
         jPanel1.add(jLTexto);
         jPanel1.add(jTTexto);
         jPanel1.add(jLTipoTecla);
         jPanel1.add(jRTipoPicto);
         jPanel1.add(jLTipoTeclaP);
         jPanel1.add(jRTipoTexto);
    
         jPanel1.add(jFuente);
         jPanel1.add(jFuenteList);

         jPanel2 = new JPanel();
         jPanel2.setLayout(new GridLayout(5, 2, 10, 10));
         jPanel2.setBorder (BorderFactory.createEmptyBorder (15, 15, 10, 15));
         jPanel2.add(jLColorTexto);
         jPanel2.add(jBTexto);
         jPanel2.add(jLColorBack);
         jPanel2.add(jBBack);
         jPanel2.add(jAncho);
         jPanel2.add(jTAncho);
         jPanel2.add(jLargo);
         jPanel2.add(jTLargo);
         jPanel2.add(jBSupr);
         jPanel2.add(jBAceptar);
         //jPanel2.add(jBCancelar);
         
         removeBottomPanel();// Quita botones Aceptar y Cancelar
         //cambiaModoSalida(JFrame.DISPOSE_ON_CLOSE);// Permite cerrar con la cruz de la esquina superior derecha
         getWindow().getContentPane().add(jPanel1, BorderLayout.NORTH); // A�ade jpanel1 arriba
         getWindow().getContentPane().add(jPanel2, BorderLayout.SOUTH);//A�ade jpanel2 debajo
    	
    }
  
 /****
     * FUNCI�N setGrupoTipo
 	 * Crea el grupo de botones para escoger el tipo de tecla (texto/picto)
 	 * PAra la ventana cuya finalidad es ser el configurador de teclas para 
 	 * tecladovirtual.
     * @author njerez
 **********************************************************************************/  
    private void  setGrupoTipo(){
    	 // ButtonGroup, jRadioButton
        jRTipoTexto = new JRadioButton(sTexto);
        jRTipoPicto = new JRadioButton(sPicto);
        jRTipoPicto.setSelected(true);
        BTipo = new ButtonGroup();
        BTipo.add(jRTipoTexto);
        BTipo.add(jRTipoPicto);
        jRTipoTexto.setFont(Global.FONT_TCV_TFIELD);
        jRTipoPicto.setFont(Global.FONT_TCV_TFIELD);
        jRTipoTexto.addActionListener(this);
        jRTipoPicto.addActionListener(this);
    }
    
 /****
     * FUNCI�N setTextFields
 	 * Crea los campos de texto y labels para Tipo de TEcla, ancho y largo.
 	 * PAra la ventana cuya finalidad es ser el configurador de teclas para 
 	 * tecladovirtual.
     * @author njerez
 **********************************************************************************/         
   private void setTextFields(){
	   jLTipoTecla = new JLabel(sTipoTecla);
       jLTipoTeclaP = new JLabel(" ");
       jLTexto = new JLabel(sITexto);
       jAncho = new JLabel (sAncho);
       jLargo = new JLabel (sLargo);
       jTTexto = new JTextField();
       
       // debug 3
       // jTLargo = new JTextField();
       // jTAncho = new JTextField();
      
       
       InternationalFormatter formato = new InternationalFormatter();
       formato.setMaximum(new Integer(600));
       formato.setMinimum(new Integer(50));
       jTLargo = new JFormattedTextField(formato);
       jTAncho = new JFormattedTextField(formato);
	/*	try {
			  mascara = new MaskFormatter("###");
			  //mascara.setValidCharacters("012345");
			  jTLargo = new JFormattedTextField(mascara);
	          //jTLargo.setValue(new Integer("###"));
	         
		
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			  mascara = new MaskFormatter("###");
			  //mascara.setValidCharacters("012345");
			 
			  jTAncho = new JFormattedTextField(mascara);
	          //jTAncho.setValue(new Integer("###"));
		
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
         */
       jTLargo.setFont(Global.FONT_TCV_TFIELD);
       jTAncho.setFont(Global.FONT_TCV_TFIELD);
       jTTexto.setFont(Global.FONT_TCV_TFIELD);
       
       
   }//fin setTExtfiels
   
   
 /****
     * FUNCI�N setBotonesColor
 	 * Crea los botones de para la elecci�n de Colores para texto y fondo.
 	 * Dise�o y funcionalidad. Y labels.
 	 * PAra la ventana cuya finalidad es ser el configurador de teclas para 
 	 * tecladovirtual.
     * @author njerez
 **********************************************************************************/      
    private void setBotonesColor(){
    	 jLColorTexto = new JLabel(sColorTexto);
         jLColorBack = new JLabel(sColorBack);
    	 ImageIcon iColor = new ImageIcon(Global.RUTA_COLOR);      
         jBTexto = new JButton(iColor);
         jBBack = new JButton(iColor);
         jBTexto.addMouseListener(new java.awt.event.MouseAdapter() {

             @Override
             public void mouseClicked(java.awt.event.MouseEvent evt) {
                 jBTextoMouseClicked(evt);
             }

             private void jBTextoMouseClicked(MouseEvent evt) {
             	Color newColor = JColorChooser.showDialog(
                         getWindow(),
                         sColorTexto,
                         cTexto);
                 	cTexto = newColor;
                 	jBTexto.setContentAreaFilled (false);
                	jBTexto.setOpaque (true);
                	jBTexto.setBackground(cTexto);
                	jBTexto.setBorder (BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
             }
         });

         jBBack.addMouseListener(new java.awt.event.MouseAdapter() {

             @Override
             public void mouseClicked(java.awt.event.MouseEvent evt) {
                 jBBackMouseClicked(evt);
             }

             private void jBBackMouseClicked(MouseEvent evt) {
                 Color newColorB = JColorChooser.showDialog(
                         getWindow(),
                         sColorBack,
                         cBack);
                 	cBack = newColorB;
                 	jBBack.setContentAreaFilled (false);
                 	jBBack.setOpaque (true);
                 	jBBack.setBackground(cBack);
                 	jBBack.setBorder (BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
             }
         });
         
    }//fin setBotonesColor
    
    
 /****
     * FUNCI�N setBotonesConf
 	 * Crea los botones de confirmaci�n: Aceptar y Suprimir. Dise�o y funcionalidad.
 	 * PAra la ventana cuya finalidad es ser el configurador de teclas para 
 	 * tecladovirtual.
     * @author njerez
 **********************************************************************************/   
    private void setBotonesConf(){
    	jBAceptar = new JButton(sAceptar);
    	jBAceptar.setFont(Global.FONT_TCV_TFIELD);
        jBAceptar.addMouseListener(new java.awt.event.MouseAdapter() {

            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jBAceptarMouseClicked(evt);
            }

            private void jBAceptarMouseClicked(MouseEvent evt) {
                 getWindow().dispose();
            }
        });
        jBCancelar = new JButton(sCancelar);
        jBCancelar.addMouseListener(new java.awt.event.MouseAdapter() {

            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jBCancelarMouseClicked(evt);
            }

            private void jBCancelarMouseClicked(MouseEvent evt) {
                 getWindow().dispose();
            }
        });
        
        jBSupr = new JButton(sSupr);
        jBSupr.setFont(Global.FONT_TCV_TFIELD);
        jBSupr.addMouseListener(new java.awt.event.MouseAdapter() {

            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jBSuprMouseClicked(evt);
            }

            private void jBSuprMouseClicked(MouseEvent evt) {
            	Object[] options = {Global.LANG.getString("si"),Global.LANG.getString("no")};
            	int n = JOptionPane.showOptionDialog(
            		    getWindow(),
            		    Global.LANG.getString("preguntaborrado"),
            		    		Global.LANG.getString("confirmborrado"),
            		    JOptionPane.YES_NO_OPTION,
            		    JOptionPane.QUESTION_MESSAGE,
            		    null,     //do not use a custom Icon
            		    options,  //the titles of buttons
            		    options[0]);
            	
            	if (n == 0){
            		bSupr = true;
                    getWindow().dispose();
            	}
            	//bSupr = true;
                //getVentana().dispose();
            }
        });
    } //fin setBotonesconf
    
    
 /****
     * FUNCI�N setFuenteList
 	 * Crea la lista de tama�os de fuente (LITTLE, NORMAL Y BIG)
 	 * Utilizado en ventana -configurdaro - tecladovirtual.
     * @author njerez
 **********************************************************************************/   
    private void setFuenteList(){
    	 jFuenteList = new JComboBox(sfont);
         jFuenteList.setSelectedIndex(1);
         jFuenteList.setFont(Global.FONT_TCV_TFIELD);
         jFuente = new JLabel (sFuente);
         //jFuente.setFont(Global.FONT_TCV_TFIELD);
    }//fin setFuenteList
        
}

	

